"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const nodeutils_1 = require("./nodeutils");
const utils_1 = require("./utils");
function createNodesCache() {
    return new WeakMap();
}
exports.default = (0, utils_1.createRule)({
    create: (ctx) => {
        const nodeCaches = {
            isWithinAsConstContext: createNodesCache(),
            isWithinRequiresAsConstContext: createNodesCache(),
            objectRequiresAsConst: createNodesCache(),
            arrayRequiresAsConst: createNodesCache()
        };
        return {
            ObjectExpression(node) {
                if (!(0, nodeutils_1.isWithinAsConstContext)(node, nodeCaches) &&
                    !(0, nodeutils_1.isWithinRequiresAsConstContext)(node, nodeCaches) &&
                    (0, nodeutils_1.objectRequiresAsConst)(node, nodeCaches)) {
                    ctx.report({
                        node,
                        messageId: "asConst",
                        fix(fixer) {
                            return fixer.insertTextAfter(node, " as const");
                        },
                    });
                }
            },
            ArrayExpression(node) {
                if (!(0, nodeutils_1.isWithinAsConstContext)(node, nodeCaches) &&
                    !(0, nodeutils_1.isWithinRequiresAsConstContext)(node, nodeCaches) &&
                    ((0, nodeutils_1.arrayRequiresAsConst)(node, nodeCaches) || (0, nodeutils_1.parentIsTrigger)(node))) {
                    ctx.report({
                        node,
                        messageId: "asConst",
                        fix(fixer) {
                            return fixer.insertTextAfter(node, " as const");
                        },
                    });
                }
            },
        };
    },
    name: "as-const",
    meta: {
        type: "problem",
        docs: {
            description: "Ensure that no 'as const' assertions are forgotten where they are necessary for correct type inference.",
            recommended: "error",
            requiresTypeChecking: true,
        },
        messages: {
            asConst: "Missing 'as const' assertion.",
        },
        fixable: "code",
        schema: [],
    },
    defaultOptions: [],
});
//# sourceMappingURL=as-const.js.map