"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parentIsTrigger = exports.arrayRequiresAsConst = exports.objectRequiresAsConst = exports.isWithinRequiresAsConstContext = exports.isWithinAsConstContext = void 0;
const utils_1 = require("@typescript-eslint/utils");
const NODES = utils_1.TSESTree.AST_NODE_TYPES;
function isWithinAsConstContext(node, nodeCaches) {
    if (nodeCaches.isWithinAsConstContext.has(node))
        return nodeCaches.isWithinAsConstContext.get(node);
    const origNode = node;
    do {
        if (node.type === NODES.TSAsExpression &&
            node.typeAnnotation.type === NODES.TSTypeReference &&
            node.typeAnnotation.typeName.type === NODES.Identifier &&
            node.typeAnnotation.typeName.name === "const") {
            nodeCaches.isWithinAsConstContext.set(origNode, true);
            return true;
        }
        if (node.type !== NODES.Property &&
            node.type !== NODES.ObjectExpression &&
            node.type !== NODES.ArrayExpression) {
            nodeCaches.isWithinAsConstContext.set(origNode, false);
            return false;
        }
    } while (node = node.parent);
    return false;
}
exports.isWithinAsConstContext = isWithinAsConstContext;
function isWithinRequiresAsConstContext(node, nodeCaches) {
    if (nodeCaches.isWithinRequiresAsConstContext.has(node))
        return nodeCaches.isWithinRequiresAsConstContext.get(node);
    const origNode = node;
    while (node = node.parent) {
        if (node.type !== NODES.Property &&
            node.type !== NODES.ObjectExpression &&
            node.type !== NODES.ArrayExpression) {
            nodeCaches.isWithinRequiresAsConstContext.set(origNode, false);
            return false;
        }
        if (requiresAsConst(node, nodeCaches, undefined)) {
            nodeCaches.isWithinRequiresAsConstContext.set(origNode, true);
            return true;
        }
    }
    return false;
}
exports.isWithinRequiresAsConstContext = isWithinRequiresAsConstContext;
function isVality(node) {
    return (node.type === NODES.Identifier &&
        (node.name === "v" || node.name === "vality"));
}
function requiresAsConst(node, nodeCaches, parentHasTrigger = false) {
    switch (node.type) {
        case NODES.ObjectExpression:
            return objectRequiresAsConst(node, nodeCaches, parentHasTrigger);
        case NODES.ArrayExpression:
            return parentHasTrigger || arrayRequiresAsConst(node, nodeCaches);
        case NODES.Literal:
            return parentHasTrigger;
        case NODES.Property:
            return requiresAsConst(node.value, nodeCaches, parentHasTrigger);
        default:
            return false;
    }
}
function objectRequiresAsConst(node, nodeCaches, parentHasTrigger = false) {
    if (nodeCaches.objectRequiresAsConst.has(node))
        return nodeCaches.objectRequiresAsConst.get(node);
    let hasTrigger = parentHasTrigger;
    if (!hasTrigger) {
        for (const prop of node.properties) {
            if (prop.type === NODES.SpreadElement)
                continue;
            if (isTrigger(prop.value)) {
                hasTrigger = true;
                break;
            }
        }
        if (!hasTrigger) {
            nodeCaches.objectRequiresAsConst.set(node, false);
            return false;
        }
        ;
    }
    for (const prop of node.properties) {
        if (prop.type === NODES.SpreadElement)
            continue;
        if (requiresAsConst(prop, nodeCaches, true)) {
            nodeCaches.objectRequiresAsConst.set(node, true);
            return true;
        }
        ;
    }
    nodeCaches.objectRequiresAsConst.set(node, false);
    return false;
}
exports.objectRequiresAsConst = objectRequiresAsConst;
function arrayRequiresAsConst(node, nodeCaches) {
    if (nodeCaches.arrayRequiresAsConst.has(node))
        return nodeCaches.arrayRequiresAsConst.get(node);
    for (const prop of node.elements) {
        if (prop.type === NODES.SpreadElement)
            continue;
        if (isTrigger(prop)) {
            return true;
        }
    }
    return false;
}
exports.arrayRequiresAsConst = arrayRequiresAsConst;
function parentIsTrigger(node) {
    return isTrigger(node.parent);
}
exports.parentIsTrigger = parentIsTrigger;
function isTrigger(node) {
    switch (node.type) {
        case NODES.ObjectExpression: {
            for (const prop of node.properties) {
                if (prop.type === NODES.SpreadElement)
                    continue;
                if (isTrigger(prop.value))
                    return true;
            }
            return false;
        }
        case NODES.ArrayExpression: {
            for (const prop of node.elements) {
                if (prop.type === NODES.SpreadElement)
                    continue;
                if (isTrigger(prop))
                    return true;
            }
            return false;
        }
        case NODES.MemberExpression:
            return isVality(node.object);
        case NODES.CallExpression:
            if (node.callee.type === NODES.MemberExpression) {
                return isVality(node.callee.object);
            }
            if (node.callee.type === NODES.CallExpression &&
                node.callee.callee.type === NODES.MemberExpression) {
                return isVality(node.callee.callee.object);
            }
        default:
            return false;
    }
}
//# sourceMappingURL=nodeutils.js.map